HITS_INFO = {
    "BM25/retrieve_results_covid_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC COVID task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_covid_top100.jsonl",
        ],
        "md5": "7dd70b911afc40647c37580e2f17a27f",
        "size (bytes)": 8779956,
        "downloaded": False,
    },
    "BM25/retrieve_results_dl19_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC DL19 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_dl19_top100.jsonl",
        ],
        "md5": "518ab9f821fb74d862589d1d141c208e",
        "size (bytes)": 2000369,
        "downloaded": False,
    },
    "BM25/retrieve_results_dl20_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC DL20 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_dl20_top100.jsonl",
        ],
        "md5": "3b6397f50fd78229e8d733243745a6d4",
        "size (bytes)": 2528494,
        "downloaded": False,
    },
    "BM25/retrieve_results_dl21_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC DL21 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_dl21_top100.jsonl",
        ],
        "md5": "6ca5e405c55fc2280d66eb0afc7416f5",
        "size (bytes)": 2140373,
        "downloaded": False,
    },
    "BM25/retrieve_results_dl22_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC DL22 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_dl22_top100.jsonl",
        ],
        "md5": "08322ee30bf084d1309f5c5a515f2f18",
        "size (bytes)": 3021942,
        "downloaded": False,
    },
    "BM25/retrieve_results_msmarco-v2.1-doc-segmented.bm25.rag24.raggy-dev_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC24-Raggy Dev task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_msmarco-v2.1-doc-segmented.bm25.rag24.raggy-dev_top100.jsonl",
        ],
        "md5": "ff8a8db2180bc628115d678135179a21",
        "size (bytes)": 26911601,
        "downloaded": False,
    },
    "BM25/retrieve_results_msmarco-v2.1-doc-segmented.bm25.rag24.researchy-dev_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC24-Researchy Dev task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_msmarco-v2.1-doc-segmented.bm25.rag24.researchy-dev_top100.jsonl",
        ],
        "md5": "41ff53ef4235f94924c75bb35f46f5f7",
        "size (bytes)": 141212740,
        "downloaded": False,
    },
    "BM25/retrieve_results_news_top100.jsonl": {
        "description": "Top-100 BM25 Results for the TREC News task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_news_top100.jsonl",
        ],
        "md5": "3fa0063db9ba1dcfc5b527a062d41720",
        "size (bytes)": 41110970,
        "downloaded": False,
    },
    "BM25/retrieve_results_noveleval_top100.jsonl": {
        "description": "Top-100 BM25 Results for the NovelEval task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25/retrieve_results_noveleval_top100.jsonl",
        ],
        "md5": "96b9e6bc7e881122ab11a94656da816e",
        "size (bytes)": 417977,
        "downloaded": False,
    },
    "BM25/trec_results_covid_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC COVID task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_covid_top100.txt",
        ],
        "md5": "ba0360f2abe60abfe361665942fbdc4b",
        "size (bytes)": 204370,
        "downloaded": False,
    },
    "BM25/trec_results_dl19_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC DL19 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_dl19_top100.txt",
        ],
        "md5": "4be11ccaf5808bd861b831b1df960907",
        "size (bytes)": 190529,
        "downloaded": False,
    },
    "BM25/trec_results_dl20_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC DL20 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_dl20_top100.txt",
        ],
        "md5": "63d07d941edc44430b3ddec3a65a0433",
        "size (bytes)": 240316,
        "downloaded": False,
    },
    "BM25/trec_results_dl21_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC DL21 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_dl21_top100.txt",
        ],
        "md5": "900915b48cbbd25b87f8cf0bfe36496f",
        "size (bytes)": 347951,
        "downloaded": False,
    },
    "BM25/trec_results_dl22_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC DL22 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_dl22_top100.txt",
        ],
        "md5": "4e93d4a41ee9343fda24887534cc0943",
        "size (bytes)": 503425,
        "downloaded": False,
    },
    "BM25/trec_results_msmarco-v2.1-doc-segmented.bm25.rag24.raggy-dev_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC24-Raggy Dev task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_msmarco-v2.1-doc-segmented.bm25.rag24.raggy-dev_top100.txt",
        ],
        "md5": "2e332446044fc6fe1197c02ddaebaab8",
        "size (bytes)": 910820,
        "downloaded": False,
    },
    "BM25/trec_results_msmarco-v2.1-doc-segmented.bm25.rag24.researchy-dev_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC24-Researchy Dev task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_msmarco-v2.1-doc-segmented.bm25.rag24.researchy-dev_top100.txt",
        ],
        "md5": "f90bf6b4a37902cd34ce1113886fd209",
        "size (bytes)": 4496144,
        "downloaded": False,
    },
    "BM25/trec_results_news_top100.txt": {
        "description": "Top-100 BM25 Results for the TREC News task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25/trec_results_news_top100.txt",
        ],
        "md5": "33e2fe3e5cd7332f0933b48516fbbe48",
        "size (bytes)": 390316,
        "downloaded": False,
    },
    "BM25_RM3/retrieve_results_covid_top100.jsonl": {
        "description": "Top-100 BM25 + RM3 Results for the TREC COVID task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25_RM3/retrieve_results_covid_top100.jsonl",
        ],
        "md5": "f80e8341da7de1d0cce166ed6e2fa84e",
        "size (bytes)": 9020100,
        "downloaded": False,
    },
    "BM25_RM3/retrieve_results_dl19_top100.jsonl": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL19 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25_RM3/retrieve_results_dl19_top100.jsonl",
        ],
        "md5": "2f8ff225e60baa2c2d47162ae4d371c1",
        "size (bytes)": 2054706,
        "downloaded": False,
    },
    "BM25_RM3/retrieve_results_dl20_top100.jsonl": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL20 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25_RM3/retrieve_results_dl20_top100.jsonl",
        ],
        "md5": "b8414ab263a05df678d375281c2bc000",
        "size (bytes)": 2548091,
        "downloaded": False,
    },
    "BM25_RM3/retrieve_results_dl21_top100.jsonl": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL21 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25_RM3/retrieve_results_dl21_top100.jsonl",
        ],
        "md5": "1a0841bb0f82b820bfb2de700a9e89b5",
        "size (bytes)": 2157834,
        "downloaded": False,
    },
    "BM25_RM3/retrieve_results_dl22_top100.jsonl": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL22 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25_RM3/retrieve_results_dl22_top100.jsonl",
        ],
        "md5": "1dd01bb2bb44b41d6af77c2879492be7",
        "size (bytes)": 3060472,
        "downloaded": False,
    },
    "BM25_RM3/retrieve_results_news_top100.jsonl": {
        "description": "Top-100 BM25 + RM3 Results for the TREC News task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/BM25_RM3/retrieve_results_news_top100.jsonl",
        ],
        "md5": "c5e8733df0a5276df2fb5b0862dba0b3",
        "size (bytes)": 38912304,
        "downloaded": False,
    },
    "BM25_RM3/trec_results_covid_top100.txt": {
        "description": "Top-100 BM25 + RM3 Results for the TREC COVID task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25_RM3/trec_results_covid_top100.txt",
        ],
        "md5": "a15d25b72b0a17b728f8215b704d7e36",
        "size (bytes)": 207451,
        "downloaded": False,
    },
    "BM25_RM3/trec_results_dl19_top100.txt": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL19 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25_RM3/trec_results_dl19_top100.txt",
        ],
        "md5": "d2d385f402ccea28f0b4b43f4d90b2af",
        "size (bytes)": 191863,
        "downloaded": False,
    },
    "BM25_RM3/trec_results_dl20_top100.txt": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL20 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25_RM3/trec_results_dl20_top100.txt",
        ],
        "md5": "de31f7400970d0472bdeaba6717c0876",
        "size (bytes)": 241312,
        "downloaded": False,
    },
    "BM25_RM3/trec_results_dl21_top100.txt": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL21 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25_RM3/trec_results_dl21_top100.txt",
        ],
        "md5": "18dd1d0be4d5126be665d48449acde02",
        "size (bytes)": 347048,
        "downloaded": False,
    },
    "BM25_RM3/trec_results_dl22_top100.txt": {
        "description": "Top-100 BM25 + RM3 Results for the TREC DL22 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25_RM3/trec_results_dl22_top100.txt",
        ],
        "md5": "071d13dab7b88ec2299232ecbb525dd9",
        "size (bytes)": 503886,
        "downloaded": False,
    },
    "BM25_RM3/trec_results_news_top100.txt": {
        "description": "Top-100 BM25 + RM3 Results for the TREC News task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/BM25_RM3/trec_results_news_top100.txt",
        ],
        "md5": "9838a6b5c067edcfeb80bf04222b0c87",
        "size (bytes)": 391639,
        "downloaded": False,
    },
    "D_BERT_KD_TASB/retrieve_results_dl19_top100.jsonl": {
        "description": "Top-100 D_BERT_KD_TASB Results for the TREC DL19 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/D_BERT_KD_TASB/retrieve_results_dl19_top100.jsonl",
        ],
        "md5": "22f14897bf37098aa4836e254287b2f1",
        "size (bytes)": 1864083,
        "downloaded": False,
    },
    "D_BERT_KD_TASB/retrieve_results_dl20_top100.jsonl": {
        "description": "Top-100 D_BERT_KD_TASB Results for the TREC DL20 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/D_BERT_KD_TASB/retrieve_results_dl20_top100.jsonl",
        ],
        "md5": "9221b308063d3cd42c574e7dbff38941",
        "size (bytes)": 2318086,
        "downloaded": False,
    },
    "D_BERT_KD_TASB/trec_results_dl19_top100.txt": {
        "description": "Top-100 D_BERT_KD_TASB Results for the TREC DL19 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/D_BERT_KD_TASB/trec_results_dl19_top100.txt",
        ],
        "md5": "8f107e6075eff57bb56e964aec2ee10a",
        "size (bytes)": 190741,
        "downloaded": False,
    },
    "D_BERT_KD_TASB/trec_results_dl20_top100.txt": {
        "description": "Top-100 D_BERT_KD_TASB Results for the TREC DL20 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/D_BERT_KD_TASB/trec_results_dl20_top100.txt",
        ],
        "md5": "9456b813de73fb68e2e12222ef9df8c4",
        "size (bytes)": 240082,
        "downloaded": False,
    },
    "OPEN_AI_ADA2/retrieve_results_dl19_top100.jsonl": {
        "description": "Top-100 OpenAI ADA2 Results for the TREC DL19 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/OPEN_AI_ADA2/retrieve_results_dl19_top100.jsonl",
        ],
        "md5": "16d5a9a1448b58d2f89c844a7c6b4a4c",
        "size (bytes)": 1849150,
        "downloaded": False,
    },
    "OPEN_AI_ADA2/retrieve_results_dl20_top100.jsonl": {
        "description": "Top-100 OpenAI ADA2 Results for the TREC DL20 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/OPEN_AI_ADA2/retrieve_results_dl20_top100.jsonl",
        ],
        "md5": "7480ae59f7146241e13ff39efb47d3f3",
        "size (bytes)": 2306232,
        "downloaded": False,
    },
    "OPEN_AI_ADA2/trec_results_dl19_top100.txt": {
        "description": "Top-100 OpenAI ADA2 Results for the TREC DL19 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/OPEN_AI_ADA2/trec_results_dl19_top100.txt",
        ],
        "md5": "63052f75197a6e01b86165d4d971f467",
        "size (bytes)": 193174,
        "downloaded": False,
    },
    "OPEN_AI_ADA2/trec_results_dl20_top100.txt": {
        "description": "Top-100 OpenAI ADA2 Results for the TREC DL20 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/OPEN_AI_ADA2/trec_results_dl20_top100.txt",
        ],
        "md5": "89482187c1cd5cc38b5288f4fd6c7369",
        "size (bytes)": 243029,
        "downloaded": False,
    },
    "REP_LLAMA/retrieve_results_dl19_top100.jsonl": {
        "description": "Top-100 repLLaMA Results for the TREC DL19 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/REP_LLAMA/retrieve_results_dl19_top100.jsonl",
        ],
        "md5": "f3cd43822547ade6602dd12af9144ebc",
        "size (bytes)": 3648493,
        "downloaded": False,
    },
    "REP_LLAMA/retrieve_results_dl20_top100.jsonl": {
        "description": "Top-100 repLLaMA Results for the TREC DL20 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/REP_LLAMA/retrieve_results_dl20_top100.jsonl",
        ],
        "md5": "e994e98c1d1f495a8a311670d5df7f07",
        "size (bytes)": 4581491,
        "downloaded": False,
    },
    "REP_LLAMA/trec_results_dl19_top100.txt": {
        "description": "Top-100 repLLaMA Results for the TREC DL19 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/REP_LLAMA/trec_results_dl19_top100.txt",
        ],
        "md5": "72adfd94f7fb04b6ae57674034cfa4e4",
        "size (bytes)": 1415843,
        "downloaded": False,
    },
    "REP_LLAMA/trec_results_dl20_top100.txt": {
        "description": "Top-100 repLLaMA Results for the TREC DL20 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/REP_LLAMA/trec_results_dl20_top100.txt",
        ],
        "md5": "5b2807d1d6f8f7f40cc7b297f9fbdcb6",
        "size (bytes)": 1781403,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_covid_top100.jsonl": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC COVID task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_covid_top100.jsonl",
        ],
        "md5": "74a41136fab99a77e8919ac29a6cc659",
        "size (bytes)": 7648893,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl19_top100.jsonl": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL19 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl19_top100.jsonl",
        ],
        "md5": "894d37a5701bde517f75e117b2d5cad4",
        "size (bytes)": 1844101,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl20_top100.jsonl": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL20 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl20_top100.jsonl",
        ],
        "md5": "241c63eba8f4ce8b7795fcaaf565bb03",
        "size (bytes)": 2285466,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl21_top100.jsonl": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL21 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl21_top100.jsonl",
        ],
        "md5": "5b90d5460014d94d977921202c42606a",
        "size (bytes)": 2082006,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl22_top100.jsonl": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL22 task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_dl22_top100.jsonl",
        ],
        "md5": "9082402af60b5e9208556853bc7842b0",
        "size (bytes)": 2958535,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_news_top100.jsonl": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC News task. (Cached JSONL format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/retrieve_results_news_top100.jsonl",
        ],
        "md5": "adcff66eb90b5f66e7ae47af7012fdb0",
        "size (bytes)": 26116086,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_covid_top100.txt": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC COVID task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_covid_top100.txt",
        ],
        "md5": "d4c49f38e7dff7dd64f3cbed7775e2fc",
        "size (bytes)": 168645,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl19_top100.txt": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL19 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl19_top100.txt",
        ],
        "md5": "0fa0c937363726dc48ee675ad53c0221",
        "size (bytes)": 147985,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl20_top100.txt": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL20 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl20_top100.txt",
        ],
        "md5": "323443d55fc9c50698d88288f53ffe8d",
        "size (bytes)": 186223,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl21_top100.txt": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL21 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl21_top100.txt",
        ],
        "md5": "dcb67a2c9cca231c28711abecb258c8a",
        "size (bytes)": 312606,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl22_top100.txt": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC DL22 task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_dl22_top100.txt",
        ],
        "md5": "8daf6f01dd39d73b9e33bf218c3b6c38",
        "size (bytes)": 454452,
        "downloaded": False,
    },
    "SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_news_top100.txt": {
        "description": "Top-100 SPLADE++ CoCondenser-EnsembleDistil Results for the TREC News task. (TREC format)",
        "urls": [
            "https://github.com/castorini/rank_llm_data/raw/main/retrieve_results/retrieve_results/SPLADE_P_P_ENSEMBLE_DISTIL/trec_results_news_top100.txt",
        ],
        "md5": "d67ff0cae7048e9cd9183c788d804296",
        "size (bytes)": 343746,
        "downloaded": False,
    },
}
